//
// $Id: NewGameController.h,v 1.6 1997/10/28 02:58:10 nygard Exp $
//

//
//  This file is a part of Empire, a game of exploration and conquest.
//  Copyright (C) 1996  Steve Nygard
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  You may contact the author by:
//     e-mail:  nygard@telusplanet.net
//

#import <AppKit/AppKit.h>
#import "Empire.h"
#import "EmpireProtocols.h" // For EmpireClientHostNotification -- should replace with NSNotificationCenter stuff.

@class Brain;
@class DistributedGameManager;
@class ServerController;

@interface NewGameController : NSObject <EmpireClientHostNotification>
{
    ServerController *serverController;

    IBOutlet NSWindow *newGamePanel;
    IBOutlet NSButton *okayButton;

    IBOutlet NSWindow *aboutPanel;
    IBOutlet NSButton *aboutPanelOkayButton;

    IBOutlet NSTextField *mapTextfield;

    IBOutlet NSTextField *player1NameTextfield;
    IBOutlet NSTextField *player2NameTextfield;
    IBOutlet NSTextField *player3NameTextfield;

    IBOutlet NSPopUpButton *player1TypePopup;
    IBOutlet NSPopUpButton *player2TypePopup;
    IBOutlet NSPopUpButton *player3TypePopup;

    IBOutlet NSMatrix *player1UnitImageMatrix;
    IBOutlet NSMatrix *player2UnitImageMatrix;
    IBOutlet NSMatrix *player3UnitImageMatrix;

    IBOutlet NSColorWell *player1ColorWell;
    IBOutlet NSColorWell *player2ColorWell;
    IBOutlet NSColorWell *player3ColorWell;

    IBOutlet NSSlider *player1ProductionEffSlider;
    IBOutlet NSSlider *player2ProductionEffSlider;
    IBOutlet NSSlider *player3ProductionEffSlider;

    IBOutlet NSSlider *player1CombatEffSlider;
    IBOutlet NSSlider *player2CombatEffSlider;
    IBOutlet NSSlider *player3CombatEffSlider;

    IBOutlet NSPopUpButton *player1RemoteClientPopup;
    IBOutlet NSPopUpButton *player2RemoteClientPopup;
    IBOutlet NSPopUpButton *player3RemoteClientPopup;

    //----------------------------------------

    IBOutlet NSWindow *newClientPanel;
    IBOutlet NSWindow *ncOkayButton;

    int ncPlayerNumber;
    IBOutlet NSBox *ncPlayerBox;
    IBOutlet NSTextField *ncNameTextfield;
    IBOutlet NSPopUpButton *ncTypePopup;
    IBOutlet NSMatrix *ncUnitImageMatrix;
    IBOutlet NSColorWell *ncColorWell;
    IBOutlet NSSlider *ncProductionEffSlider;
    IBOutlet NSSlider *ncCombatEffSlider;

    DistributedGameManager *clientGameManager;
}

- (void) awakeFromNib;

- initWithBrain:(Brain *)theBrain;
- (void) dealloc;

- (void) updatePlayer1Color:sender;
- (void) updatePlayer2Color:sender;
- (void) updatePlayer3Color:sender;

- (void) createNewGame:sender;

- (void) aboutPlayer1:sender;
- (void) aboutPlayer2:sender;
- (void) aboutPlayer3:sender;
- (void) aboutStopAction:sender;

- (void) setMapPanel:sender;

- (void) showPanel;
- (void) choosePlayer:(Player)number forGameManager:(DistributedGameManager *)gameManager;
- (void) createNewClient:sender;

//======================================================================
// Empire Image Vendor Notification
//======================================================================

- (void) vendorImagesUpdated:(BOOL)player1:(BOOL)player2:(BOOL)player3:(BOOL)other;

//======================================================================
// Window Delegate
//======================================================================

- (void) windowDidBecomeKey:(NSNotification *)notification;
  
//======================================================================
// Client Host Notification
//======================================================================

- (void) newClientHost:(NSString *)hostname;
- (void) removedClientHostNumber:(int)index;

@end
